@@ -338,18 +338,16 @@ sub is_empty
     ! @{ $$listh }
 }
 
-sub data
+sub node_data
 {
     my $listh   = shift;
     my $node    = $$listh;
 
     # return the existing data, sans the next ref.
 
-    my @valz    = @{ $node }[ 1 .. $#$node ];
-
     wantarray
-    ?  @valz
-    : \@valz
+    ?   @{ $node }[ 1 .. $#$node ]
+    : [ @{ $node }[ 1 .. $#$node ] ]
 }
 
 sub next_data
@@ -403,6 +401,24 @@ sub clear_data
     $listh
 }
 
+sub list_data
+{
+    my $listh   = shift;
+    my $node    = $$listh;
+
+    my @return  = ();
+
+    while( @$node )
+    {
+        my ( $node, @data ) = @$node;
+        push @return, \@data;
+    }
+
+    wantarray
+    ?  @return
+    : \@return
+}
+
 ########################################################################
 # access the list head.
 #
@@ -448,8 +464,6 @@ sub head
 
 sub new_head
 {
-$DB::single = 1;
-
     my ( $listh, $head ) = @_;
 
     my $root    = $listh->root_node;
@@ -549,8 +563,7 @@ sub cut
 
     if( defined wantarray )
     {
-        my @valz   =  @{ $node->[0] };
-        $node->[0] = shift @valz;
+        ( $node->[0], my @valz ) = @{ $node->[0] };
 
         wantarray
         ?  @valz
@@ -572,57 +585,70 @@ sub cut
 sub splice
 {
     my $listh   = shift;
-    my $count   = shift || 1;
+    my $count   = shift || 0;
 
     looks_like_number $count
-    or croak "Bogus splice: non-numeric '$count'";
+    or croak "Bogus splice: non-numeric count '$count'";
 
-    $count > 0
-    or croak "Bogus splice: negative count '$count'";
+    $count < 0
+    and croak "Bogus splice: negative count '$count'";
+
+    # short circut if there is nothing to do.
+
+    $count > 0 || @_ or return;
 
     my $node    = $$listh
     or confess "Bogus splice: empty list handler";
 
-    my $tail    = $node;
+    my $dead    = '';
 
-    for( 1 .. $count )
+    if( $count > 0 )
     {
-        @$tail or last;
+        my $tail    = $node;
 
-        $tail   = $tail->[0];
+        for( 1 .. $count )
+        {
+            @$tail or last;
+
+            $tail   = $tail->[0];
+        }
+
+        # this is the start of the chain that gets removed.
+        # keep it alive for a few steps to see if the caller
+        # wants it back or we should clean it up.
+        #
+        # after that, splice the node out of the list.
+
+        $dead       = $node->[0];
+        $node->[0]  = delete $tail->[0];
+        $tail->[0]  = [];
     }
 
-    # this is the start of the chain that gets removed.
-    # keep it alive for a few steps to see if the caller
-    # wants it back or we should clean it up.
-    #
-    # after that, splice the node out of the list.
-
-    my $dead    = $node->[0];
-    $node->[0]  = delete $tail->[0];
-
-    # at this point $dead is a runt linked
-    # list without a terminating node.
+    # at this point $dead is either false or 
+    # a runt linked list lacking its terminating
+    # node.
     #
     # insert anything on the stack after the
     # current node.
 
     for( @_ )
     {
-       $node    = $node->[0] = [ $node->[0], shift ];
+       $node    = $node->[0] = [ $node->[0], $_ ];
     }
 
+    # nothing to return or clean up if there 
+    # wasn't anything removed.
+
+    $dead or return;
+
     # if the caller wants anything back then
     # clean up the dead chain and hand it back.
     #
-    # node: maybe this should return an array of
-    # arrayrefs?
+    # alternative: array of data?
 
     if( defined wantarray )
     {
         # hand back a linked list with $dead as the head node.
-
-        $tail->[0]  = [];
         
         my $new     = $listh->new;
 
@@ -630,15 +656,16 @@ sub splice
 
         @$head      = @$dead;
 
-        $new
+        return $new
     }
     else
     {
-        # discard the dead links.
-
         $cleanup->( $dead );
+
+        return
     }
 }
+
 ########################################################################
 # aside: push can be very expensive.
 # but, then, so is maintaining a separate
@@ -741,7 +768,7 @@ LinkedList::Single - singly linked list manager.
 
     # extract the data from the current node.
 
-    my @data    = $listh->data;
+    my @data    = $listh->node_data;
 
     # save and restore a node position
 
@@ -823,7 +850,7 @@ LinkedList::Single - singly linked list manager.
 
         my $altlist = $listh->clone;
 
-        my @data    = $altlist->next->data;
+        my @data    = $altlist->next->node_data;
 
         ...
     }
@@ -871,7 +898,7 @@ LinkedList::Single - singly linked list manager.
         # deal with @data
     }
 
-    # note that $listh->next->data may be empty
+    # note that $listh->next->node_data may be empty
     # even if there are mode nodes due to a node
     # having no data.
 
@@ -977,7 +1004,7 @@ into a single method because there isn't any clean way
 to determine if the node needs to be emptied or left
 unmodified due to an empty stack. The option of using
 
-    $listh->data( undef )
+    $listh->node_data( undef )
 
 for cleaning the node leaves no way to store an explicit
 undef in the node.
